@@ -162,13 +162,16 @@ function mt:remove()
     self._gc:remove()
 end
 
+---@async
 function mt:notify(method, params)
     proto.doMethod {
         method = method,
         params = params,
     }
+    await.sleep(0.1)
 end
 
+---@async
 function mt:request(method, params, callback)
     local id = counter()
     self._waiting[id] = {
@@ -181,16 +184,20 @@ function mt:request(method, params, callback)
         method = method,
         params = params,
     }
+    await.sleep(0.1)
 end
 
 ---@async
 function mt:awaitRequest(method, params)
     return await.wait(function (waker)
-        self:request(method, params, function (result)
-            if result == json.null then
-                result = nil
-            end
-            waker(result)
+        ---@async
+        await.call(function ()
+            self:request(method, params, function (result)
+                if result == json.null then
+                    result = nil
+                end
+                waker(result)
+            end)
         end)
     end)
 end
