@@ -9,11 +9,12 @@ local fsu     = require 'fs-utility'
 local define  = require "proto.define"
 local files   = require 'files'
 local await   = require 'await'
-local timer   = require 'timer'
 local encoder = require 'encoder'
 local ws      = require 'workspace.workspace'
 local scope   = require 'workspace.scope'
 local inspect = require 'inspect'
+local jsonb   = require 'json-beautify'
+local jsonc   = require 'jsonc'
 
 local m = {}
 
@@ -21,7 +22,7 @@ m.metaPaths = {}
 
 local function getDocFormater(uri)
     local version = config.get(uri, 'Lua.runtime.version')
-    if client.isVSCode() then
+    if client.getOption('viewDocument') then
         if version == 'Lua 5.1' then
             return 'HOVER_NATIVE_DOCUMENT_LUA51'
         elseif version == 'Lua 5.2' then
@@ -50,7 +51,7 @@ end
 
 local function convertLink(uri, text)
     local fmt = getDocFormater(uri)
-    return text:gsub('%$([%.%w]+)', function (name)
+    return text:gsub('%$([%.%w_%:]+)', function (name)
         local lastDot = ''
         if name:sub(-1) == '.' then
             name = name:sub(1, -2)
@@ -80,7 +81,7 @@ local function createViewDocument(name)
     if not fmt then
         return nil
     end
-    name = name:match '[%w_%.]+'
+    name = name:match '[%w_%.%:]+'
     if name:sub(-1) == '.' then
         name = name:sub(1, -2)
     end
@@ -187,10 +188,14 @@ local function compileSingleMetaDoc(uri, script, metaLang, status)
     if not suc then
         log.debug('MiddleScript:\n', middleScript)
     end
+    local text = table.concat(compileBuf)
     if disable and status == 'default' then
-        return nil
+        return text, false
     end
-    return table.concat(compileBuf)
+    if status == 'disable' then
+        return text, false
+    end
+    return text, true
 end
 
 local function loadMetaLocale(langID, result)
@@ -209,6 +214,7 @@ local function initBuiltIn(uri)
     local langID   = lang.id
     local version  = config.get(uri, 'Lua.runtime.version')
     local encoding = config.get(uri, 'Lua.runtime.fileEncoding')
+    ---@type fs.path
     local metaPath = fs.path(METAPATH) / config.get(uri, 'Lua.runtime.meta'):gsub('%$%{(.-)%}', {
         version  = version,
         language = langID,
@@ -220,11 +226,8 @@ local function initBuiltIn(uri)
         loadMetaLocale(langID, metaLang)
     end
 
-    if scp:get('metaPath') == metaPath:string() then
-        log.debug('Has meta path, skip:', metaPath:string())
-        return
-    end
-    scp:set('metaPath', metaPath:string())
+    local metaPaths = {}
+    scp:set('metaPaths', metaPaths)
     local suc = xpcall(function ()
         if not fs.exists(metaPath) then
             fs.create_directories(metaPath)
@@ -239,18 +242,31 @@ local function initBuiltIn(uri)
     for libName, status in pairs(define.BuiltIn) do
         status = config.get(uri, 'Lua.runtime.builtin')[libName] or status
         log.debug('Builtin status:', libName, status)
-        if status == 'disable' then
-            goto CONTINUE
-        end
-        libName = libName .. '.lua'
-        local libPath = templateDir / libName
-        local metaDoc = compileSingleMetaDoc(uri, fsu.loadFile(libPath), metaLang, status)
+
+        ---@type fs.path
+        local libPath = templateDir / (libName .. '.lua')
+        local metaDoc, include = compileSingleMetaDoc(uri, fsu.loadFile(libPath), metaLang, status)
         if metaDoc then
             metaDoc = encoder.encode(encoding, metaDoc, 'auto')
-            out:saveFile(libName, metaDoc)
-            local outputPath = metaPath / libName
+
+            local outputLibName = libName:gsub('%.', '/') .. '.lua'
+            if outputLibName ~= libName then
+                out:createDirectories(fs.path(outputLibName):parent_path())
+            end
+
+            local ok, err = out:saveFile(outputLibName, metaDoc)
+            if not ok then
+                log.debug("Save Meta File Failed:", err)
+                goto CONTINUE
+            end
+
+            local outputPath = metaPath / outputLibName
             m.metaPaths[outputPath:string()] = true
             log.debug('Meta path:', outputPath:string())
+
+            if include then
+                metaPaths[#metaPaths+1] = outputPath:string()
+            end
         end
         ::CONTINUE::
     end
@@ -260,16 +276,80 @@ local function initBuiltIn(uri)
     end
 end
 
-local function loadSingle3rdConfig(libraryDir)
-    local configText = fsu.loadFile(libraryDir / 'config.lua')
+---@param libraryDir fs.path
+---@return table?
+local function loadSingle3rdConfigFromJson(libraryDir)
+    local path = libraryDir / 'config.json'
+    local configText = fsu.loadFile(path)
+    if not configText then
+        return nil
+    end
+
+    local suc, cfg = xpcall(jsonc.decode_jsonc, function (err)
+        log.error('Decode config.json failed at:', libraryDir:string(), err)
+    end, configText)
+    if not suc then
+        return nil
+    end
+
+    if type(cfg) ~= 'table' then
+        log.error('config.json must be an object:', libraryDir:string())
+        return nil
+    end
+
+    return cfg
+end
+
+---@param libraryDir fs.path
+---@return table?
+local function loadSingle3rdConfigFromLua(libraryDir)
+    local path = libraryDir / 'config.lua'
+    local configText = fsu.loadFile(path)
     if not configText then
         return nil
     end
 
     local env = setmetatable({}, { __index = _G })
-    assert(load(configText, '@' .. libraryDir:string(), 't', env))()
+    local f, err = load(configText, '@' .. libraryDir:string(), 't', env)
+    if not f then
+        log.error('Load config.lua failed at:', libraryDir:string(), err)
+        return nil
+    end
+
+    local suc = xpcall(f, function (err)
+        log.error('Load config.lua failed at:', libraryDir:string(), err)
+    end)
+
+    if not suc then
+        return nil
+    end
 
     local cfg = {}
+    for k, v in pairs(env) do
+        cfg[k] = v
+    end
+
+    return cfg
+end
+
+---@param libraryDir fs.path
+local function loadSingle3rdConfig(libraryDir)
+    local cfg = loadSingle3rdConfigFromJson(libraryDir)
+    if not cfg then
+        cfg = loadSingle3rdConfigFromLua(libraryDir)
+        if not cfg then
+            return
+        end
+        local jsonbuf = jsonb.beautify(cfg)
+        client.requestMessage('Info', lang.script.WINDOW_CONFIG_LUA_DEPRECATED, {
+            lang.script.WINDOW_CONVERT_CONFIG_LUA,
+        }, function (action, index)
+            if index == 1 and jsonbuf then
+                fsu.saveFile(libraryDir / 'config.json', jsonbuf)
+                fsu.fileRemove(libraryDir / 'config.lua')
+            end
+        end)
+    end
 
     cfg.path = libraryDir:filename():string()
     cfg.name = cfg.name or cfg.path
@@ -278,23 +358,19 @@ local function loadSingle3rdConfig(libraryDir)
         cfg.plugin = true
     end
 
-    for k, v in pairs(env) do
-        cfg[k] = v
-    end
-
     if cfg.words then
         for i, word in ipairs(cfg.words) do
-            cfg.words[i] = '()' .. word .. '()'
+            cfg.words[i] = '([%w_]?)' .. word .. '([%w_]?)'
         end
     end
     if cfg.files then
         for i, filename in ipairs(cfg.files) do
-            if plat.OS == 'Windows' then
+            if plat.os == 'windows' then
                 filename = filename:gsub('/', '\\')
             else
                 filename = filename:gsub('\\', '/')
             end
-            cfg.files[i] = '()' .. filename .. '()'
+            cfg.files[i] = '([%w_]?)' .. filename .. '([%w_]?)'
         end
     end
 
@@ -321,7 +397,13 @@ local function load3rdConfigInDir(dir, configs, inner)
 end
 
 local function load3rdConfig(uri)
-    local configs = {}
+    local scp = scope.getScope(uri)
+    local configs = scp:get 'thirdConfigsCache'
+    if configs then
+        return configs
+    end
+    configs = {}
+    scp:set('thirdConfigsCache', configs)
     load3rdConfigInDir(innerThirdDir, configs, true)
     local thirdDirs = config.get(uri, 'Lua.workspace.userThirdParty')
     for _, thirdDir in ipairs(thirdDirs) do
@@ -332,15 +414,37 @@ end
 
 local function apply3rd(uri, cfg, onlyMemory)
     local changes = {}
-    if cfg.configs then
-        for _, change in ipairs(cfg.configs) do
-            changes[#changes+1] = {
-                key    = change.key,
-                action = change.action,
-                prop   = change.prop,
-                value  = change.value,
-                uri    = uri,
-            }
+    if cfg.settings then
+        for key, value in pairs(cfg.settings) do
+            if type(value) == 'table' then
+                if #value == 0 then
+                    for k, v in pairs(value) do
+                        changes[#changes+1] = {
+                            key    = key,
+                            action = 'prop',
+                            prop   = k,
+                            value  = v,
+                            uri    = uri,
+                        }
+                    end
+                else
+                    for _, v in ipairs(value) do
+                        changes[#changes+1] = {
+                            key    = key,
+                            action = 'add',
+                            value  = v,
+                            uri    = uri,
+                        }
+                    end
+                end
+            else
+                changes[#changes+1] = {
+                    key    = key,
+                    action = 'set',
+                    value  = value,
+                    uri    = uri,
+                }
+            end
         end
     end
 
@@ -363,43 +467,47 @@ local function apply3rd(uri, cfg, onlyMemory)
     client.setConfig(changes, onlyMemory)
 end
 
-local hasAsked
+local hasAsked = {}
 ---@async
-local function askFor3rd(uri, cfg)
-    if hasAsked then
+local function askFor3rd(uri, cfg, checkThirdParty)
+    if hasAsked[cfg.name] then
         return nil
     end
-    hasAsked = true
-    local yes1 = lang.script.WINDOW_APPLY_WHIT_SETTING
-    local yes2 = lang.script.WINDOW_APPLY_WHITOUT_SETTING
-    local no   = lang.script.WINDOW_DONT_SHOW_AGAIN
-    local result = client.awaitRequestMessage('Info'
-        , lang.script('WINDOW_ASK_APPLY_LIBRARY', cfg.name)
-        , {yes1, yes2, no}
-    )
-    if not result then
-        return nil
-    end
-    if result == yes1 then
+
+    if checkThirdParty == 'Apply' then
         apply3rd(uri, cfg, false)
-        client.setConfig({
-            {
-                key    = 'Lua.workspace.checkThirdParty',
-                action = 'set',
-                value  = false,
-                uri    = uri,
-            },
-        }, false)
-    elseif result == yes2 then
+    elseif checkThirdParty == 'ApplyInMemory' then
         apply3rd(uri, cfg, true)
-        client.setConfig({
-            {
-                key    = 'Lua.workspace.checkThirdParty',
-                action = 'set',
-                value  = false,
-                uri    = uri,
-            },
-        }, true)
+    elseif checkThirdParty == 'Disable' then
+        return nil
+    elseif checkThirdParty == 'Ask' then
+        hasAsked[cfg.name] = true
+        local applyAndSetConfig = lang.script.WINDOW_APPLY_WHIT_SETTING
+        local applyInMemory = lang.script.WINDOW_APPLY_WHITOUT_SETTING
+        local dontShowAgain   = lang.script.WINDOW_DONT_SHOW_AGAIN
+        local result = client.awaitRequestMessage('Info'
+            , lang.script('WINDOW_ASK_APPLY_LIBRARY', cfg.name)
+            , {applyAndSetConfig, applyInMemory, dontShowAgain}
+        )
+        if not result then
+            -- "If none got selected"
+            -- See: https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#window_showMessageRequest
+            return nil
+        end
+        if result == applyAndSetConfig then
+            apply3rd(uri, cfg, false)
+        elseif result == applyInMemory then
+            apply3rd(uri, cfg, true)
+        else
+            client.setConfig({
+                {
+                    key    = 'Lua.workspace.checkThirdParty',
+                    action = 'set',
+                    value  = 'Disable',
+                    uri    = uri,
+                },
+            }, false)
+        end
     end
 end
 
@@ -407,114 +515,153 @@ end
 ---@param b string
 ---@return boolean
 local function wholeMatch(a, b)
-    local pos1, pos2 = a:match(b)
-    if not pos1 then
-        return false
-    end
-    local left  = a:sub(pos1 - 1, pos1 - 1)
-    local right = a:sub(pos2, pos2)
-    if left:match '[%w_]'
-    or right:match '[%w_]' then
-        return false
-    end
-    return true
+    local captures = {
+        a:match(b),
+    }
+    return captures[1] == '' and captures[#captures] == ''
 end
 
-local function check3rdByWords(uri, text, configs)
-    if hasAsked then
+local function check3rdByWords(uri, configs, checkThirdParty)
+    if not files.isLua(uri) then
         return
     end
+    local id = 'check3rdByWords:' .. uri
+    await.close(id)
     await.call(function () ---@async
+        await.sleep(0.1)
+        local text = files.getText(uri)
+        if not text then
+            return
+        end
         for _, cfg in ipairs(configs) do
-            if cfg.words then
-                for _, word in ipairs(cfg.words) do
-                    await.delay()
-                    if wholeMatch(text, word) then
-                        askFor3rd(uri, cfg)
-                        return
-                    end
+            if not cfg.words then
+                goto CONTINUE
+            end
+            if hasAsked[cfg.name] then
+                goto CONTINUE
+            end
+            local library = ('%s/library'):format(cfg.dirname)
+            if util.arrayHas(config.get(uri, 'Lua.workspace.library'), library) then
+                goto CONTINUE
+            end
+            for _, word in ipairs(cfg.words) do
+                await.delay()
+                if wholeMatch(text, word) then
+                    log.info('Found 3rd library by word: ', word, uri, library, inspect(config.get(uri, 'Lua.workspace.library')))
+                    ---@async
+                    await.call(function ()
+                        askFor3rd(uri, cfg, checkThirdParty)
+                    end)
+                    return
                 end
             end
+            ::CONTINUE::
         end
-    end)
+    end, id)
 end
 
-local function check3rdByFileName(uri, configs)
-    if hasAsked then
-        return
-    end
+local function check3rdByFileName(uri, configs, checkThirdParty)
     local path = ws.getRelativePath(uri)
     if not path then
         return
     end
+    local id = 'check3rdByFileName:' .. uri
+    await.close(id)
     await.call(function () ---@async
+        await.sleep(0.1)
         for _, cfg in ipairs(configs) do
-            if cfg.files then
-                for _, filename in ipairs(cfg.files) do
-                    await.delay()
-                    if wholeMatch(path, filename) then
-                        askFor3rd(uri, cfg)
-                        return
-                    end
+            if not cfg.files then
+                goto CONTINUE
+            end
+            if hasAsked[cfg.name] then
+                goto CONTINUE
+            end
+            local library = ('%s/library'):format(cfg.dirname)
+            if util.arrayHas(config.get(uri, 'Lua.workspace.library'), library) then
+                goto CONTINUE
+            end
+            for _, filename in ipairs(cfg.files) do
+                await.delay()
+                if wholeMatch(path, filename) then
+                    log.info('Found 3rd library by filename: ', filename, uri, library, inspect(config.get(uri, 'Lua.workspace.library')))
+                    ---@async
+                    await.call(function ()
+                        askFor3rd(uri, cfg, checkThirdParty)
+                    end)
+                    return
                 end
             end
+            ::CONTINUE::
         end
-    end)
+    end, id)
 end
 
-local lastCheckedUri = {}
-local function checkedUri(uri)
-    if  lastCheckedUri[uri]
-    and timer.clock() - lastCheckedUri[uri] < 5 then
-        return false
-    end
-    lastCheckedUri[uri] = timer.clock()
-    return true
-end
-
-local thirdConfigs
+---@async
 local function check3rd(uri)
-    if hasAsked then
+    if ws.isIgnored(uri) then
         return
     end
-    if not config.get(uri, 'Lua.workspace.checkThirdParty') then
+    local checkThirdParty = config.get(uri, 'Lua.workspace.checkThirdParty')
+    -- Backwards compatability: `checkThirdParty` used to be a boolean.
+    if not checkThirdParty or checkThirdParty == 'Disable' then
+        return
+    elseif checkThirdParty == true then
+        checkThirdParty = 'Ask'
+    end
+    local scp = scope.getScope(uri)
+    if not scp:get 'canCheckThirdParty' then
         return
     end
-    if thirdConfigs == nil then
-        thirdConfigs = load3rdConfig(uri) or false
-    end
+    local thirdConfigs = load3rdConfig(uri) or false
     if not thirdConfigs then
         return
     end
-    if checkedUri(uri) then
-        if files.isLua(uri) then
-            local text = files.getText(uri)
-            if text then
-                check3rdByWords(uri, text, thirdConfigs)
-            end
+    check3rdByWords(uri, thirdConfigs, checkThirdParty)
+    check3rdByFileName(uri, thirdConfigs, checkThirdParty)
+end
+
+local function check3rdOfWorkspace(suri)
+    local scp = scope.getScope(suri)
+    scp:set('thirdConfigsCache', nil)
+    scp:set('canCheckThirdParty', true)
+    local id = 'check3rdOfWorkspace:' .. scp:getName()
+    await.close(id)
+    ---@async
+    await.call(function ()
+        ws.awaitReady(suri)
+        for uri in files.eachFile(suri) do
+            check3rd(uri)
         end
-        check3rdByFileName(uri, thirdConfigs)
-    end
+        for uri in files.eachDll() do
+            check3rd(uri)
+        end
+    end, id)
 end
 
 config.watch(function (uri, key, value, oldValue)
     if key:find '^Lua.runtime' then
         initBuiltIn(uri)
     end
+    if key == 'Lua.workspace.checkThirdParty'
+    or key == 'Lua.workspace.userThirdParty'
+    or key == '' then
+        check3rdOfWorkspace(uri)
+    end
 end)
 
+---@async
 files.watch(function (ev, uri)
     if ev == 'update'
     or ev == 'dll' then
+        await.sleep(1)
         check3rd(uri)
     end
 end)
 
-function m.init()
-    initBuiltIn(nil)
-    for _, scp in ipairs(ws.folders) do
-        initBuiltIn(scp.uri)
+ws.watch(function (ev, uri)
+    if ev == 'startReload' then
+        initBuiltIn(uri)
     end
-end
+end)
 
 return m
